%least_squares_map
%
% Andy French June 2013

function least_squares_map

%% INPUTS %%

%Create random data based upon average line y = m*x + c
N = 100;
m = 0.6;
c = 2;
xmin = 0;
xmax = 10;

%Characterize normally distributed deviations from (x,y)
sigma_x = 0.5;
sigma_y = 0.5;

%%

%Create data vectors
delta_x = sigma_x*randn(1,N);
delta_y = sigma_y*randn(1,N);
x = linspace( xmin, xmax, N ) + delta_x;
y = m*x + c + delta_y ;

%Compute line of best fit coefficients from regression formula
[mvert,cvert] = LSfit_vert(x,y);
[mhorz,chorz] = LSfit_horz(x,y);

%Determine product moment correlation coefficient
p = cv(x,y)/( sqrt(var(x)) * sqrt(var(y)) );

%Compute line of best fit coefficients from minima of surface of sum of
%square y deviations
a = linspace( 0, 5, N );
b = linspace( -2, 2, N );
[a,b] = meshgrid(a,b);
S = zeros(N,N);
for n=1:N
    S = S + ( a + b*x(n) - y(n) ).^2 ;
end

%Determine indices of array S which correspond to global minima
[i,j] = find(S==min(min(S)));

%Plot surface of square deviations from line of best fit vs best fit
%parameters
figure('color',[1 1 1]);
surf(a,b,S);
hold on;
plot3( a(i,j),b(i,j),S(i,j),'w*')
shading interp
xlabel('a')
ylabel('b')
zlabel('Sum of square y deviations from y = a + b*x')
title('Sum of squares of y deviations from y = a + b*x')
view(2);
colorbar;
print(gcf,'-dpng','-r300','surface of square errors 2D')
view(3)
camorbit(150,0);
print(gcf,'-dpng','-r300','surface of square errors 3D')
cla;

%Plot line of best fit
view(2);
plot(x,y,'r+');
hold on;
plot( x, a(i,j) + b(i,j)*x,'b')
plot( x, cvert + mvert*x,'g')
plot( x, chorz + mhorz*x,'m')
xlims = get(gca,'xlim');
ylims = get(gca,'ylim');
xlabel('x')
ylabel('y')
title({'Least squares linear regression',...
    ['Vertical: y = ',num2str(cvert), ' + ',num2str(mvert),'x.'],...
    ['Horizontal: y = ',num2str(chorz), ' + ',num2str(mhorz),'x.'],...
    ['Correlation p = ',num2str(p) ] }); 
legend({'Data','LS regression from surface (vert)','LS regression from formula (vert)','LS regression from formula (horz)'})
print(gcf,'-dpng','-r300','linear regression')
close(gcf);

%Export data as an Excel file
data = cell(1:length(x),2);
data{1,1} = 'x';
data{1,2} = 'y';
for n=1:length(x)
    data{1+n,1} = x(n);
    data{1+n,2} = y(n);
end
xlswrite('least_squares_data',data);

%%

%Line of best fit y = mx + c minimizing sum of squares of y deviations
function [m,c] = LSfit_vert(x,y)
m = cv(x,y)/var(x);
c = mean(y) - m * mean(x);

%%

%Line of best fit y = mx + c minimizing sum of squares of x deviations
function [m,c] = LSfit_horz(x,y)

%Work out line of best fit of the for x = m*y + c
M = cv(x,y)/var(y);
C = mean(x) - M * mean(y);

%Rearrange to form y = x/M -C/M = m*x + c
m = 1/M;
c = -C/M;

%%

%Covariance of data x and y
function c = cv(x,y)
N = length(x);
c = sum(x.*y)/N - mean(x)*mean(y);


%End of code

